@@ -1,10 +1,10 @@
 use crate::arg;
 use crate::args;
 use crate::args::ArgSpec;
-use cdx::agg::{AggList, AggMaker};
+use cdx::agg::{AggMaker, LineAggList};
 use cdx::column::{ColumnHeader, Writer};
 use cdx::comp::{CompMaker, LineComp, LineCompList};
-use cdx::util::{err, get_writer, Error, LookbackReader, Result, TextLine};
+use cdx::util::{err, get_writer, Error, Reader, Result, TextLine};
 use std::cmp::Ordering;
 use std::io::Write;
 
@@ -171,7 +171,7 @@ pub fn main(argv: &[String]) -> Result<()> {
     ];
     let (args, files) = args::parse(&prog, &A, argv);
 
-    let mut agg = AggList::new();
+    let mut agg = LineAggList::new();
     let mut comp = LineCompList::new();
     let mut count = Count::default();
 
@@ -198,7 +198,7 @@ pub fn main(argv: &[String]) -> Result<()> {
 
     assert_eq!(files.len(), 1);
 
-    let mut f = LookbackReader::new(1);
+    let mut f = Reader::new();
     f.open(&files[0])?;
     if f.is_empty() {
         return Ok(());
@@ -238,7 +238,7 @@ pub fn main(argv: &[String]) -> Result<()> {
         return Ok(());
     }
 
-    f.do_split = comp.need_split();
+    f.do_split(comp.need_split());
     let mut matches = 1;
     if !agg.is_empty() {
         agg.add(f.curr_line());
