@@ -1,7 +1,7 @@
 use crate::arg;
 use crate::args;
 use crate::args::ArgSpec;
-use cdx::agg::{AggMaker, AggWholeList};
+use cdx::agg::{AggMaker, AggerList};
 use cdx::column::{ColumnHeader, Writer};
 use cdx::num;
 use cdx::util::{get_writer, Reader, Result, StringLine, TextLine, Tri};
@@ -9,16 +9,15 @@ use std::io::Write;
 
 /*
 
---columns =>treat each column as a separate named file. Merge if multiple files.
 --report-every-million-input-lines
 
 */
 struct NamedAgg {
     name: String,
-    agg: AggWholeList,
+    agg: AggerList,
 }
 impl NamedAgg {
-    fn new(name: &str, agg: AggWholeList) -> Self {
+    fn new(name: &str, agg: AggerList) -> Self {
         Self {
             name: name.to_string(),
             agg,
@@ -43,7 +42,7 @@ pub fn main(argv: &[String]) -> Result<()> {
     ];
     let (args, files) = args::parse(&prog, &A, argv);
 
-    let mut agg = AggWholeList::new();
+    let mut agg = AggerList::new();
     let mut file_name_col = "file".to_string();
     let mut show_file_name = Tri::Maybe;
     let mut show_header = Tri::Maybe;
@@ -88,35 +87,18 @@ pub fn main(argv: &[String]) -> Result<()> {
             unreachable!();
         }
     }
-    /*
-        let show_file = match show_file_name {
-            Tri::Yes => true,
-            Tri::No => false,
-            Tri::Maybe => files.len() > 1 && !total_only,
-        };
-        let do_totals = match show_totals {
-            Tri::Yes => true,
-            Tri::No => false,
-            Tri::Maybe => files.len() > 1,
-        } || total_only;
-        let do_header = match show_header {
-            Tri::Yes => true,
-            Tri::No => false,
-            Tri::Maybe => agg.len() > 1 || show_file,
-        };
-    */
     let nada = TextLine::new();
     let mut w = get_writer("-")?;
     let mut first_file = true;
     let mut totals = Vec::new();
     let do_totals;
     let show_file;
+    agg.fmt(fmt);
     if do_columns {
         if agg.is_empty() {
             agg.push("bytes,asum,chars")?;
         }
         totals.resize(agg.len(), 0.0);
-        agg.fmt(fmt);
         let mut aggs: Vec<NamedAgg> = Vec::new();
         let mut colmap: Vec<usize> = Vec::new();
         for x in &files {
@@ -146,9 +128,6 @@ pub fn main(argv: &[String]) -> Result<()> {
                 }
             }
         }
-        for x in &mut aggs {
-            x.agg.fmt(fmt);
-        }
         show_file = match show_file_name {
             Tri::Yes => true,
             Tri::No => false,
@@ -184,7 +163,7 @@ pub fn main(argv: &[String]) -> Result<()> {
                     if i != 0 {
                         w.write_all(b"\t")?;
                     }
-                    x.agg.get(i).agg.borrow_mut().result(&mut w)?;
+                    x.agg.get(i).agg.borrow_mut().result(&mut w, fmt)?;
                 }
                 w.write_all(b"\n")?;
             }
@@ -202,7 +181,6 @@ pub fn main(argv: &[String]) -> Result<()> {
             agg.push("lines,count")?;
         }
         totals.resize(agg.len(), 0.0);
-        agg.fmt(fmt);
         show_file = match show_file_name {
             Tri::Yes => true,
             Tri::No => false,
@@ -242,7 +220,7 @@ pub fn main(argv: &[String]) -> Result<()> {
                 continue;
             }
 
-            f.do_split = false;
+            f.do_split(false);
             loop {
                 agg.add(&f.curr_line().line);
                 if f.getline()? {
