@@ -8,12 +8,12 @@
 //!
 //! ```
 //! use cdx::comp::LineCompList;
-//! use cdx::util::LookbackReader;
+//! use cdx::util::Reader;
 //! let mut comp = LineCompList::new();
 //! comp.add("price,float")?;
 //! comp.add("quant,num")?;
 //! let input = "<< CDX\tname\tquant\tprice\naaa\t42\t1.23\nbbb\t12\t2.34\nccc\t12\t2.34\n";
-//! let mut reader = LookbackReader::new(2);
+//! let mut reader = Reader::new_with(2);
 //! reader.open(input);
 //! comp.lookup(&reader.names())?;
 //! reader.getline()?;
@@ -27,7 +27,7 @@ use crate::column::{get_col, NamedCol};
 use crate::expr::Expr;
 use crate::num::{fcmp, ulp_to_ulong, Junk, JunkType, JunkVal};
 use crate::text::Text;
-use crate::util::{err, prerr_n, Error, LookbackReader, Result, TextLine};
+use crate::util::{err, prerr_n, Error, Reader, Result, TextLine};
 use lazy_static::lazy_static;
 use std::cmp::Ordering;
 use std::fmt;
@@ -875,6 +875,9 @@ impl CompMaker {
         Self::do_push("length", "Sort by length of string", |_p| {
             Ok(Box::new(CompareLen::new()))
         })?;
+        Self::do_push("random", "Sort randomly", |_p| {
+            Ok(Box::new(CompareRandom::new()))
+        })?;
         Self::do_push("ip", "Sort as IP address or 1.2.3 section numbers", |_p| {
             Ok(Box::new(CompareIP::new()))
         })?;
@@ -1451,6 +1454,39 @@ impl LineCompList {
     }
 }
 
+#[derive(Default, Debug)]
+struct CompareRandom {
+    rng: fastrand::Rng,
+}
+impl CompareRandom {
+    fn new() -> Self {
+        Self::default()
+    }
+    fn ord(&self) -> Ordering {
+        if self.rng.bool() {
+            Ordering::Less
+        } else {
+            Ordering::Greater
+        }
+    }
+}
+impl Compare for CompareRandom {
+    fn comp(&self, _left: &[u8], _right: &[u8]) -> Ordering {
+        self.ord()
+    }
+    fn equal(&self, _left: &[u8], _right: &[u8]) -> bool {
+        self.rng.bool()
+    }
+    fn fill_cache(&self, _item: &mut Item, _value: &[u8]) {}
+    fn set(&mut self, _value: &[u8]) {}
+    fn comp_self(&self, _right: &[u8]) -> Ordering {
+        self.ord()
+    }
+    fn equal_self(&self, _right: &[u8]) -> bool {
+        self.rng.bool()
+    }
+}
+
 /// IP Address Comparison
 #[derive(Default, Debug)]
 struct CompareIP {
@@ -1875,7 +1911,7 @@ impl LineCompare for LineCompExpr {
 
 /// return true if ordering is bad.
 /// print appropriate message to stderr
-pub fn comp_check(f: &LookbackReader, cmp: &mut LineCompList, unique: bool) -> bool {
+pub fn comp_check(f: &Reader, cmp: &mut LineCompList, unique: bool) -> bool {
     let c = cmp.comp_cols(f.prev_line(1), f.curr_line());
     let bad = match c {
         Ordering::Less => false,
