@@ -1,5 +1,6 @@
 //! Numeric Helpers
 
+use crate::text::Text;
 use crate::util::{err, Error, Result};
 use std::cmp::Ordering;
 use std::io::Write;
@@ -99,9 +100,9 @@ pub fn ulp_to_ulong(d: f64) -> u64 {
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub enum NumFormat {
     /// as integer
-    Plain,
+    Plain(usize),
     /// 1.2e34
-    Float,
+    Float(usize),
     /// power of 2, e.g. 3K
     Power2,
     /// power of 10 e.g. 3k
@@ -109,41 +110,83 @@ pub enum NumFormat {
 }
 impl Default for NumFormat {
     fn default() -> Self {
-        Self::Plain
+        Self::Plain(0)
     }
 }
 impl NumFormat {
     /// new from string
     pub fn new(spec: &str) -> Result<Self> {
-        if spec.eq_ignore_ascii_case("plain")
-            || spec.eq_ignore_ascii_case("int")
-            || spec.eq_ignore_ascii_case("integer")
-        {
-            Ok(Self::Plain)
-        } else if spec.eq_ignore_ascii_case("float") || spec.eq_ignore_ascii_case("exp") {
-            Ok(Self::Float)
+        if spec.eq_ignore_ascii_case("plain") {
+            Ok(Self::Plain(0))
+        } else if spec.eq_ignore_ascii_case("float") {
+            Ok(Self::Float(0))
         } else if spec.eq_ignore_ascii_case("power2") || spec.eq_ignore_ascii_case("p2") {
             Ok(Self::Power2)
         } else if spec.eq_ignore_ascii_case("power10") || spec.eq_ignore_ascii_case("p10") {
             Ok(Self::Power10)
+        } else if let Some((a, b)) = spec.split_once('.') {
+            let p = b.to_usize_whole(spec.as_bytes(), "Num Format")?;
+            if a.eq_ignore_ascii_case("plain") {
+                Ok(Self::Plain(p))
+            } else if a.eq_ignore_ascii_case("float") {
+                Ok(Self::Float(p))
+            } else {
+                err!("Number format must be plain[.N], float[.N], power2 or power10")
+            }
         } else {
-            err!("Number format must be plain, float, power1 or power10")
+            err!("Number format must be plain[.N], float[.N], power2 or power10")
         }
     }
 }
 
+const P2_LETTERS: &[u8] = b"0KMGTPEZY";
+const P10_LETTERS: &[u8] = b"0kmgtpezy";
+const P2_VALUES_U: [usize; 9] = [
+    1,
+    1024,
+    1024 ^ 2,
+    1024 ^ 3,
+    1024 ^ 4,
+    1024 ^ 5,
+    1024 ^ 6,
+    usize::MAX,
+    usize::MAX,
+];
+const P2_VALUES_F: [f64; 9] = [
+    1.0,
+    1024.0,
+    1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0,
+    1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0,
+];
+
 /// format a number
 pub fn format_hnum(mut num: f64, fmt: NumFormat, mut w: impl Write) -> Result<()> {
-    if fmt == NumFormat::Plain || num.abs() < 1000.0 {
-        write!(w, "{}", num)?;
+    if let NumFormat::Plain(n) = fmt {
+        if n == 0 {
+            write!(w, "{num}")?;
+        } else {
+            write!(w, "{num:.0$}", n)?;
+        }
         return Ok(());
     }
-    if fmt == NumFormat::Float {
-        write!(w, "{:.2e}", num)?;
+    if let NumFormat::Float(n) = fmt {
+        if n == 0 {
+            write!(w, "{num:e}")?;
+        } else {
+            write!(w, "{num:.0$e}", n)?;
+        }
+        return Ok(());
+    }
+    num = num.round();
+    if num.abs() < 1000.0 || num.is_nan() || num.is_infinite() {
+        write!(w, "{num}")?;
         return Ok(());
     }
-    const P2_LETTERS: &[u8] = b"0KMGTPEZY";
-    const P10_LETTERS: &[u8] = b"0kmgtpezy";
     let sign = if num < 0.0 {
         num = -num;
         "-"
@@ -190,3 +233,22 @@ pub fn format_hnum(mut num: f64, fmt: NumFormat, mut w: impl Write) -> Result<()
         }
     }
 }
+
+/// return value associated with suffix character, e.g. K returns 1024 and k returns 1000
+pub fn suffix_valf(ch: u8) -> Option<f64> {
+    for (i, x) in P2_LETTERS.iter().enumerate() {
+        if *x == ch {
+            return Some(P2_VALUES_F[i]);
+        }
+    }
+    None
+}
+/// return value associated with suffix character, e.g. K returns 1024 and k returns 1000
+pub fn suffix_valu(ch: u8) -> Option<usize> {
+    for (i, x) in P2_LETTERS.iter().enumerate() {
+        if *x == ch {
+            return Some(P2_VALUES_U[i]);
+        }
+    }
+    None
+}
